//
//  StarfieldView.m
//  CoreAnimationStarfield
//
//  Created by Patrick Geiller on 28/03/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "StarfieldView.h"


static double frandom(double start, double end)
{
  double r = random();
  r /= RAND_MAX;
  r = start + r*(end-start);
  
  return r;
}




@implementation StarfieldView

//
//	awakeFromNib
//		build starfield layer and set bloom
//
- (void)awakeFromNib
{
	// Setup core animation
	[self setWantsLayer:true];
	CALayer* rootLayer = [self layer];

	// Create our container layer
	starfieldLayer = [CALayer layer];
	[self orientWithX:0 andY:0.3];
	[rootLayer addSublayer:starfieldLayer];
	// Center layer
	[self viewDidEndLiveResize];

	[self buildStars];

	[self addBloom];

	[self spinLayer:[starfieldLayer.sublayers objectAtIndex:150]];

}


//
//	buildStars
//		create 3d text layers
//
- (void)buildStars
{

	// Common text style
	NSDictionary* textStyle = [NSDictionary dictionaryWithObjectsAndKeys:
	@"Futura-MediumItalic", @"font",
	kCAAlignmentCenter, @"alignmentMode",
	nil];

	// Range of textlayer positions
	float	width	= [self frame].size.width/1.5;
	float	height	= [self frame].size.height/1.5;

	int i, numStars = 200;
	for (i=0; i<numStars; i++)
	{
		
		float x		= frandom(-width, width);
		float y		= frandom(-height, height);

		float red	= frandom(0.0, 0.7);
		float green	= frandom(0.6, 0.8);
		float blue	= 1;
		
		float fontSize	= frandom(10, 20);

		// Create text layer
		CATextLayer* layer = [CATextLayer layer];
		layer.fontSize = fontSize;
		layer.style = textStyle;
		layer.foregroundColor = CGColorCreateGenericRGB(red, green, blue, 1);
		layer.position = CGPointMake(x, y);

		// Randomly select its content
		float whichText = frandom(0, 3);
		if (whichText >= 0.0 && whichText <= 1.0)	layer.string = @"Core";
		if (whichText >= 1.0 && whichText <= 2.0)	layer.string = @"Animation";
		if (whichText >= 2.0 && whichText <= 3.0)	layer.string = @"Starfield";

		// Set layer bounds
		CGSize s = [layer preferredFrameSize];
		layer.bounds = CGRectMake(0, 0, s.width, s.height);


		//
		// We animate position and opacity. 
		//	position is just going from A to B,
		//	opacity is going from invisble to visible to invisble, therefore we need a keyframe animation.
		//
		float	duration = frandom(0.8, 8);
			
		CABasicAnimation *theAnimation;
		theAnimation=[CABasicAnimation animationWithKeyPath:@"zPosition"];
		theAnimation.fromValue=[NSNumber numberWithFloat:frandom(800, 400)];
		theAnimation.toValue=[NSNumber numberWithFloat:frandom(-300, -100)];
		theAnimation.duration=duration;
		theAnimation.repeatCount = 1e100;
		[layer addAnimation:theAnimation forKey:@"zPosition"];

		CAKeyframeAnimation* opacityAnimation;
		opacityAnimation = [CAKeyframeAnimation animationWithKeyPath:@"opacity"];
		opacityAnimation.values = [NSArray arrayWithObjects:
									[NSNumber numberWithFloat:0.0], 
									[NSNumber numberWithFloat:1.0], 
									[NSNumber numberWithFloat:1.0], 
									[NSNumber numberWithFloat:0.0],
									nil];
		opacityAnimation.duration = duration;
		opacityAnimation.repeatCount = 1e100;
		[layer addAnimation:opacityAnimation forKey:@"opacity"];
		 
		[starfieldLayer addSublayer:layer];
	}
}


//
//	addBloom
//		add a filter on the starfield layer itself
//
- (void)addBloom
{
	CIFilter* filter = [CIFilter filterWithName:@"CIBloom"];
    [filter setDefaults];
    [filter setValue:[NSNumber numberWithFloat:10.0] forKey:@"inputRadius"];
    [filter setValue:[NSNumber numberWithFloat:2.0] forKey:@"inputIntensity"];
	starfieldLayer.filters = [NSArray arrayWithObject:filter];
}


//
// Recenter starfield on resize
//
- (void)viewDidEndLiveResize
{
	starfieldLayer.position = CGPointMake([self frame].size.width/2, [self frame].size.height/2);
}



//
//	orientWithX andY
//		set a new perspective transform for starfieldLayer, built from 2 rotation angles
//
- (void)orientWithX:(float)x andY:(float)y
{
	CATransform3D transform = CATransform3DMakeRotation(x, 0, 1, 0); 
	transform = CATransform3DRotate(transform, y, 1, 0, 0);
	float zDistance = -450; 
	transform.m34 = 1.0 / -zDistance;
	starfieldLayer.sublayerTransform = transform;	
}


//
//	IB : button toggle bloom
//
- (IBAction)toggleBloom:(id)sender
{
	if ([starfieldLayer.filters count])
	{
		starfieldLayer.filters = nil;
	}
	else
		[self addBloom];

}


//
//	IB : button reset orientation (reset X and Y rotation)
//
- (IBAction)resetOrientation:(id)sender
{
	angleX = 0;
	angleY = 0;

	// Set custom duration for animating to new orientation
	[CATransaction begin];
	[CATransaction setValue:[NSNumber numberWithFloat:3] forKey:kCATransactionAnimationDuration];

	[self orientWithX:0 andY:0];
	
	[CATransaction commit];
}


//
//	IB : remove rotation animations from text layers
//
- (IBAction)resetClickedLayers:(id)sender
{

	for (CALayer* layer in starfieldLayer.sublayers)
	{
		[layer removeAnimationForKey:@"rotationX"];
		[layer removeAnimationForKey:@"rotationZ"];
	}
}


//
//	mouseDown, mouseDragged, mouseUp
//		on mouseDragged, change orientation
//		on mouseUp, spin layer if we didn't change orientation
- (void)mouseDown:(NSEvent *)theEvent
{
	draggedDuringThisClick = false;
	// Get view coordinates
	dragStart = NSPointToCGPoint([self convertPoint:theEvent.locationInWindow fromView:nil]);
}

- (void)mouseDragged:(NSEvent *)theEvent
{
	// Get view coordinates
	CGPoint point = NSPointToCGPoint([self convertPoint:theEvent.locationInWindow fromView:nil]);
	draggedDuringThisClick = true;
	
	deltaX = (point.x - dragStart.x)/200;
	deltaY = -(point.y - dragStart.y)/200;
	[self orientWithX:(angleX+deltaX) andY:(angleY+deltaY)];
}

- (void)mouseUp:(NSEvent *)theEvent
{
	if (draggedDuringThisClick)	
	{
		angleX += deltaX;
		angleY += deltaY;
		return;
	}

	// Get view coordinates
	CGPoint point = NSPointToCGPoint([self convertPoint:theEvent.locationInWindow fromView:nil]);
	CALayer* layer = [[self layer] hitTest:point];
	
	// Only spin text layers
	if (strcmp(object_getClassName(layer), "CATextLayer"))	return;

	CATextLayer* textLayer = (CATextLayer*)layer;
	[self spinLayer:textLayer];
}

//
//	spinLayer
//		spin a text layer around x and z
//
- (void)spinLayer:(CATextLayer*)textLayer
{
	textLayer.foregroundColor = CGColorCreateGenericRGB(1, 0.3, 0.5, 1);

	CABasicAnimation *theAnimation;
	 
	theAnimation=[CABasicAnimation animationWithKeyPath:@"transform.rotation.x"];
	theAnimation.fromValue=[NSNumber numberWithFloat:0];
	theAnimation.toValue=[NSNumber numberWithFloat:-6000];
	theAnimation.duration=1000;
	theAnimation.repeatCount = 1e100;
	[textLayer addAnimation:theAnimation forKey:@"rotationX"];

	theAnimation=[CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
	theAnimation.fromValue=[NSNumber numberWithFloat:0];
	theAnimation.toValue=[NSNumber numberWithFloat:-3000];
	theAnimation.duration=800;
	theAnimation.repeatCount = 1e100;
	[textLayer addAnimation:theAnimation forKey:@"rotationZ"];
}

- (BOOL)mouseDownCanMoveWindow
{
	return	NO;
}




@end
